Attribute VB_Name = "TransPbmp"
'This is a essentially a straight 32-bit port of the code in the
'original MS KB article with one modification that allows the
'destination for the bitmap to a Form as well as a PictureBox.
'The code could be further modified to support other custom
'objects that directly support the hDC property. The companion
'module TransBmpH.bas contains a variation of this code that
'takes this a step further by accepting an Hwnd for a destination
'and using the GetDC API to obtain the hDC directly.

Option Explicit

Private Type BITMAP
    bmType As Long
    bmWidth As Long
    bmHeight As Long
    bmWidthBytes As Long
    bmPlanes As Integer
    bmBitsPixel As Integer
    bmBits As Long
End Type

Private Declare Function BitBlt Lib "gdi32" (ByVal srchDC As Long, ByVal srcX As Long, ByVal srcY As Long, ByVal srcW As Long, ByVal srcH As Long, ByVal desthDC As Long, ByVal destX As Long, ByVal destY As Long, ByVal op As Long) As Long
Private Declare Function SetBkColor Lib "gdi32" (ByVal hdc As Long, ByVal crColor As Long) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function CreateBitmap Lib "gdi32" (ByVal nWidth As Long, ByVal nHeight As Long, ByVal nPlanes As Long, ByVal nBitCount As Long, lpBits As Any) As Long
Private Declare Function CreateCompatibleBitmap Lib "gdi32" (ByVal hdc As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function GetObj Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, bmp As Any) As Long

Private Const SRCCOPY = &HCC0020
Private Const SRCAND = &H8800C6
Private Const SRCPAINT = &HEE0086
Private Const NOTSRCCOPY = &H330008

Public Sub TransparentBlt(dest As Object, ByVal srcBmp As Long, _
ByVal destX As Long, ByVal destY As Long, ByVal TransColor As Long)
Attribute TransparentBlt.VB_Description = "dest must be Form or PicBox. srcBmp is .Picture property of Form, PicBox, or Image. TransColor is bkgrd color of bmp to be transparent."

'TransparentBlt() accepts six parameters: a destination picture or form
'   (dest), a source bitmap to become transparent (srcBmp), the X,Y
'   coordinates in pixels where you want to place the source bitmap on the
'   destination (destX and destY), and the RGB value for the color you want
'   to be transparent. TransparentBlt() copies the source bitmap to any X,Y
'   location on the background making areas transparent.
'===================================================================
'ADDITIONAL NOTES:
'The source (srcBmp) does NOT have to be visible.
'(srcBmp) can be .Picture property of PicureBox, Form, or Image control
'**OR** the Handle property of an OLE Picture or StdPicture object.
'   With the Handle of a an OLE Picture/StdPicture, the bitmap can be loaded
'   from a RES file without tying up picturebox or image object resources.
'   To use OLE Picutre Objects, a reference to
'   "Standard OLE Types" must be added to the project.
'(dest) can be ANY object for which you can obtain a Hwnd. This would
'include VB objects that support the Hwnd property or any window for
'that matter.

'If the destination is a Form or PictureBox and its AutoRedraw
'property is True, then a Refresh of the destination may be
'necessary for the result to become visible.

    Dim destScale As Integer
    Dim srcDC As Long  'source bitmap (color)
    Dim saveDC As Long 'backup copy of source bitmap
    Dim maskDC As Long 'mask bitmap (monochrome)
    Dim invDC As Long  'inverse of mask bitmap (monochrome)
    Dim resultDC As Long 'combination of source bitmap & background
    Dim bmp As BITMAP 'description of the source bitmap
    Dim hResultBmp As Long 'Bitmap combination of source & background
    Dim hSaveBmp As Long 'Bitmap stores backup copy of source bitmap
    Dim hMaskBmp As Long 'Bitmap stores mask (monochrome)
    Dim hInvBmp As Long  'Bitmap holds inverse of mask (monochrome)
    Dim hPrevBmp As Long 'Bitmap holds previous bitmap selected in DC
    Dim hSrcPrevBmp As Long  'Holds previous bitmap in source DC
    Dim hSavePrevBmp As Long 'Holds previous bitmap in saved DC
    Dim hDestPrevBmp As Long 'Holds previous bitmap in destination DC
    Dim hMaskPrevBmp As Long 'Holds previous bitmap in the mask DC
    Dim hInvPrevBmp As Long 'Holds previous bitmap in inverted mask DC
    Dim OrigColor As Long 'Holds original background color from source DC
    Dim Success As Long 'Stores result of call to Windows API
    '*** The following line was modified to include Form object. ***
    If (TypeOf dest Is PictureBox) Or (TypeOf dest Is Form) Then
        destScale = dest.ScaleMode 'Store ScaleMode to restore later
        dest.ScaleMode = vbPixels 'Set ScaleMode to pixels for Windows GDI
        'Retrieve bitmap to get width (bmp.bmWidth) & height (bmp.bmHeight)
        Success = GetObj(srcBmp, Len(bmp), bmp)
        srcDC = CreateCompatibleDC(dest.hdc)    'Create DC to hold stage
        saveDC = CreateCompatibleDC(dest.hdc)   'Create DC to hold stage
        maskDC = CreateCompatibleDC(dest.hdc)   'Create DC to hold stage
        invDC = CreateCompatibleDC(dest.hdc)    'Create DC to hold stage
        resultDC = CreateCompatibleDC(dest.hdc) 'Create DC to hold stage
        'Create monochrome bitmaps for the mask-related bitmaps:
        hMaskBmp = CreateBitmap(bmp.bmWidth, bmp.bmHeight, 1, 1, ByVal 0&)
        hInvBmp = CreateBitmap(bmp.bmWidth, bmp.bmHeight, 1, 1, ByVal 0&)
        'Create color bitmaps for final result & stored copy of source
        hResultBmp = CreateCompatibleBitmap(dest.hdc, bmp.bmWidth, bmp.bmHeight)
        hSaveBmp = CreateCompatibleBitmap(dest.hdc, bmp.bmWidth, bmp.bmHeight)
        hSrcPrevBmp = SelectObject(srcDC, srcBmp)     'Select bitmap in DC
        hSavePrevBmp = SelectObject(saveDC, hSaveBmp) 'Select bitmap in DC
        hMaskPrevBmp = SelectObject(maskDC, hMaskBmp) 'Select bitmap in DC
        hInvPrevBmp = SelectObject(invDC, hInvBmp)    'Select bitmap in DC
        hDestPrevBmp = SelectObject(resultDC, hResultBmp) 'Select bitmap
        'Make backup of source bitmap to restore later
        Success = BitBlt(saveDC, 0, 0, bmp.bmWidth, bmp.bmHeight, srcDC, 0, 0, SRCCOPY)
        'Create mask: set background color of source to transparent color.
        OrigColor = SetBkColor(srcDC, TransColor)
        Success = BitBlt(maskDC, 0, 0, bmp.bmWidth, bmp.bmHeight, srcDC, 0, 0, SRCCOPY)
        TransColor = SetBkColor(srcDC, OrigColor)
        'Create inverse of mask to AND w/ source & combine w/ background.
        Success = BitBlt(invDC, 0, 0, bmp.bmWidth, bmp.bmHeight, maskDC, 0, 0, NOTSRCCOPY)
        'Copy background bitmap to result & create final transparent bitmap
        Success = BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight, dest.hdc, destX, destY, SRCCOPY)
        'AND mask bitmap w/ result DC to punch hole in the background by
        'painting black area for non-transparent portion of source bitmap.
        Success = BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight, maskDC, 0, 0, SRCAND)
        'AND inverse mask w/ source bitmap to turn off bits associated
        'with transparent area of source bitmap by making it black.
        Success = BitBlt(srcDC, 0, 0, bmp.bmWidth, bmp.bmHeight, invDC, 0, 0, SRCAND)
        'XOR result w/ source bitmap to make background show through.
        Success = BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight, srcDC, 0, 0, SRCPAINT)
        Success = BitBlt(dest.hdc, destX, destY, bmp.bmWidth, bmp.bmHeight, resultDC, 0, 0, SRCCOPY) 'Display transparent bitmap on backgrnd
        Success = BitBlt(srcDC, 0, 0, bmp.bmWidth, bmp.bmHeight, saveDC, 0, 0, SRCCOPY) 'Restore backup of bitmap.
        hPrevBmp = SelectObject(srcDC, hSrcPrevBmp) 'Select orig object
        hPrevBmp = SelectObject(saveDC, hSavePrevBmp) 'Select orig object
        hPrevBmp = SelectObject(resultDC, hDestPrevBmp) 'Select orig object
        hPrevBmp = SelectObject(maskDC, hMaskPrevBmp) 'Select orig object
        hPrevBmp = SelectObject(invDC, hInvPrevBmp) 'Select orig object
        Success = DeleteObject(hSaveBmp)   'Deallocate system resources.
        Success = DeleteObject(hMaskBmp)   'Deallocate system resources.
        Success = DeleteObject(hInvBmp)    'Deallocate system resources.
        Success = DeleteObject(hResultBmp) 'Deallocate system resources.
        Success = DeleteDC(srcDC)          'Deallocate system resources.
        Success = DeleteDC(saveDC)         'Deallocate system resources.
        Success = DeleteDC(invDC)          'Deallocate system resources.
        Success = DeleteDC(maskDC)         'Deallocate system resources.
        Success = DeleteDC(resultDC)       'Deallocate system resources.
        dest.ScaleMode = destScale 'Restore ScaleMode of destination.
    End If
End Sub

